; Project name:	Any2DB
; Version:	1.4
; Author:	Alexander Shmonov
; Contacts:	shmonov@yahoo.com

.686
.mmx
.model		flat,stdcall
option		casemap	:none
option		scoped

include		d:\masm32\include\windows.inc
include		d:\masm32\include\user32.inc
include		d:\masm32\include\kernel32.inc
include		d:\masm32\include\masm32.inc

includelib	d:\masm32\lib\user32.lib
includelib	d:\masm32\lib\kernel32.lib
includelib	d:\masm32\lib\masm32.lib

;===============================================================	EQUATES
inv	equ	invoke

;===============================================================	STRUCTURES
Buf		STRUCT
		p	dd	?
		l	dd	?
Buf		ENDS

;===============================================================	MACROS
;Name:		pt
;Description:	makes procedure prototype declaration clear and short
;In:	name	procedure name
;	count	number of parameters
;Out:		declares a prototype of <name> procedure

pt		MACRO	name,count
	LOCAL	result
		result	equ < PROTO>
		IF	count GT 0
		result	CATSTR result,< :DWORD>
		REPEAT	count-1
		result	CATSTR result,<,:DWORD>
		ENDM
		ENDIF
		name	result
		ENDM
;-------------------------------

;Name:		m2m
;Description:	copies a dword value from one memory location to another
;In:	M1	destination address
;	M2	source address
;Out:	M1	contains a value of M2

m2m		MACRO	M1,M2
		push	M2
		pop	dword ptr M1
		ENDM
;-------------------------------

;Name:		return
;Description:	returns to a caller with a return value in eax
;In:	Arg	return value
;Out:	eax	contains a value of Arg

return		MACRO	Arg
		mov	eax,Arg
		ret
		ENDM
;-------------------------------

;Name:		FatalIf
;Description:	if eax==val, the macro calls ErrFatal
;In:	hWin	window handle
;	val	"fatal" value
;	pMsg	address of a message to display
;Out:	none

FatalIf		MACRO	hWin,val,pMsg
		cmp	eax,val
		jne	@f
		inv	ErrFatal,hWin,pMsg
	@@:
		ENDM
;-------------------------------

;Name:		FatalIfEx
;Description:	if eax==val, the macro calls ErrFatalEx
;In:	hWin	window handle
;	val	"fatal" value
;	pMsg1	pointer to "error type" message
;	pMsg2	pointer to "error source" message
;Out:	none

FatalIfEx	MACRO	hWin,val,pMsg1,pMsg2
		cmp	eax,val
		jne	@f
		inv	ErrFatalEx,hWin,pMsg1,pMsg2
	@@:
		ENDM
;-------------------------------

;===============================================================	PROTOTYPES
pt	AllocEx,3
pt	Concat,2
pt	ErrFatal,2
pt	ErrFatalEx,3
pt	PrepareErrorMsg,2
pt	GetCmdLineParams,2
;===============================================================	INITIALIZED DATA
.data
eFatal		db	"Fatal Error",0
eOpen		db	"Can't open ",0
eRead		db	"Can't read ",0
eWrite		db	"Can't write ",0
exBufSrc	db	"source file",0
exBufTgt	db	"target file",0
sErrCode	db	"Error code: ",0
eMemAlloc	db	"Memory allocation error",0
eMemAlloc2	db	"Can't allocate memory for the ",0
sCaption	db	"About",0
sUsage		db	"Any2DB version 1.4f",13,10
		db	" 2006 Alexander Shmonov, shmonov@yahoo.com",13,10
		db	13,10
		db	"Converts data from any file to assembly language file",13,10
		db	"containing the data declared with DB directive",13,10
		db	"preceded by a variable name. Usage:",13,10
		db	13,10
		db	"any2db <source file> <target file> <variable name> <style>",13,10
		db	"<style> defines either of the two styles:",13,10
		db	"	h or H for 0##h",13,10
		db	"	x or X for 0x##"
n		db	0
style		db	0	;hex output style

;===============================================================	UNINITIALIZED DATA
.data?
BufErrMsg	Buf	<>	;error messages buffer
BufSrc		Buf	<>	;source file buffer
BufTgt		Buf	<>	;target file buffer
hfSrc		dd	?	;source file handle
hfTgt		dd	?	;target file handle
hInstance	dd	?	;application instance
nRead		dd	?	;number of bytes actually read
psParam1	dd	?	;parameter #1
psParam2	dd	?	;parameter #2
psParam3	dd	?	;parameter #3
psParam4	dd	?	;parameter #4

;===============================================================	CODE
.code
Start:		inv	GetModuleHandle,0
		mov	hInstance,eax
		inv	VirtualAlloc,0,4096,MEM_RESERVE or MEM_COMMIT,PAGE_READWRITE
		FatalIf 0,0,ADDR eMemAlloc
		mov	BufErrMsg.p,eax					;error message buffer
		inv	GetCmdLineParams,4,ADDR psParam1
		cmp	al,4
		je	Proceed
Help:		inv	MessageBox,0,ADDR sUsage,ADDR sCaption,MB_OK or MB_ICONINFORMATION
		jmp	Exit
Proceed:	mov	eax,psParam4
		mov	al,byte ptr [eax]
		cmp	al,'H'
		je	StyleH
		cmp	al,'h'
		je	StyleH
		cmp	al,'X'
		je	StyleX
		cmp	al,'x'
		jne	Help
StyleX:		inc	style
StyleH:		mov	eax,psParam3
	@@lp:	inc	eax
		cmp	byte ptr [eax],0
		jne	@@lp
		sub	eax,psParam3
		push	eax
		inv	CreateFile,psParam1,GENERIC_READ,0,0,OPEN_ALWAYS,FILE_ATTRIBUTE_NORMAL,0
		FatalIfEx 0,INVALID_HANDLE_VALUE,ADDR eOpen,psParam1
		mov	hfSrc,eax
		inv	GetFileSize,eax,0
		mov	BufSrc.l,eax
		inv	AllocEx,0,ADDR BufSrc,ADDR exBufSrc
		mov	ecx,eax
		and	ecx,0fh	;remainder
		cmp	ecx,0
		je	@@skip
		lea	ecx,[ecx*4+ecx+4]
	@@skip:	shr	eax,4	;number of full rows
		imul	eax,84	;each row is 84 bytes in both cases
		add	eax,ecx	;now, plus the trail
		pop	ecx	;length of the var name
		inc	ecx	;plus tab char
		add	eax,ecx	;here's the full length!
		mov	BufTgt.l,eax
		
		inv	AllocEx,0,ADDR BufSrc,ADDR exBufSrc
		
		inv	ReadFile,hfSrc,BufSrc.p,BufSrc.l,ADDR nRead,0
		FatalIfEx 0,INVALID_HANDLE_VALUE,ADDR eRead,psParam1
		
		inv	AllocEx,0,ADDR BufTgt,ADDR exBufTgt
		inv	CreateFile,psParam2,GENERIC_WRITE,0,0,CREATE_ALWAYS,FILE_ATTRIBUTE_NORMAL,0
		FatalIfEx 0,INVALID_HANDLE_VALUE,ADDR eOpen,psParam2
		mov	hfTgt,eax

	;processing the file
		push	edi
		mov	edx,BufTgt.p
		mov	ebx,psParam3
		movzx	edi,style
nextvarch:	mov	al,byte ptr [ebx]
		cmp	al,0
		je	vartab
		mov	byte ptr [edx],al
		inc	edx
		inc	ebx
		jmp	nextvarch
vartab:		mov	byte ptr [edx],20h
		mov	ecx,nRead
		mov	ebx,BufSrc.p
		inc	edx
NextLine:	mov	word ptr [edx],6264h
		inc	edx
		inc	edx
		mov	byte ptr [edx],9
		inc	edx
NextByte:	movzx	ax,byte ptr [ebx]
		inc	ebx
		ror	ax,4
		ror	ah,4
		add	ax,3030h
		cmp	al,3Ah
		jb	DontAdjAL
		add	al,7
DontAdjAL:	cmp	ah,3Ah
		jb	DontAdjAH
		add	ah,7
DontAdjAH:	cmp	edi,0
		je	H1
		mov	word ptr [edx],7830h
		inc	edx
		inc	edx
		jmp	X1
	H1:	mov	byte ptr [edx],30h
		inc	edx
	X1:	mov	word ptr [edx],ax
		inc	edx
		inc	edx
		cmp	edi,0
		jne	X2
		mov	byte ptr [edx],68h
		inc	edx
	X2:	inc	n
		cmp	n,16
		je	newline
		mov	byte ptr [edx],2Ch
		inc	edx
		loop	NextByte
		dec	edx
		jmp	Done
newline:	mov	n,0
		mov	word ptr [edx],0A0Dh
		inc	edx
		inc	edx
		loop	NextLine
		dec	edx
		dec	edx
Done:		sub	edx,BufTgt.p
		pop	edi
	;writing the file
		inv	WriteFile,hfTgt,BufTgt.p,edx,ADDR nRead,0
		FatalIfEx 0,INVALID_HANDLE_VALUE,ADDR eWrite,psParam2

Exit:		inv	ExitProcess,0

;---------------------------------------------------------------	PROCEDURES

;Name:		ErrFatal
;Description:	displays a message box with error description, then terminates the program
;In:	hWin	handle to the current window
;	pMsg	pointer to the message to display
;Out:	none
ErrFatal	proc	hWin,pMsg
		inv	MessageBox,hWin,pMsg,ADDR eFatal,MB_OK or MB_ICONERROR
		inv	ExitProcess,-1
ErrFatal	endp
;-------------------------------

;Name:		ErrFatalEx
;Description:	displays a message box with extended error description, then terminates the program
;In:	hWin	handle to the current window
;	pMsg1	pointer to "error type" message
;	pMsg2	pointer to "error source" message
;Out:	none
ErrFatalEx	proc	hWin,pMsg1,pMsg2
		inv	PrepareErrorMsg,pMsg1,pMsg2
		inv	MessageBox,hWin,BufErrMsg.p,ADDR eFatal,MB_OK or MB_ICONERROR
		inv	ExitProcess,-1
ErrFatalEx	endp
;-------------------------------

;Name:		PrepareErrorMsg
;Description:	creates formatted error message at [BufErrMsg]
;In:	pMsg1	pointer to "error type" message
;	pMsg2	pointer to "error source" message
;Out:	none
PrepareErrorMsg	proc	pMsg1,pMsg2
		pushfd
		push	ecx
		push	edx
		push	edi
	;clearing the error message buffer
		mov	edi,BufErrMsg.p
		push	edi
		mov	ecx,1024
		xor	eax,eax
		cld
		rep	stosd
		pop	eax
	;concatenating strings at [pErrMsg]
		inv	Concat,eax,pMsg1
		inv	Concat,eax,pMsg2
		mov	word ptr [eax],0A0Dh
		inc	eax
		inc	eax
		push	eax
		call	GetLastError
		pop	ecx
		push	eax
		push	ecx
		inv	FormatMessage,FORMAT_MESSAGE_FROM_SYSTEM,0,eax,0,ecx,2000,0
		pop	ecx
		add	eax,ecx
		inv	Concat,eax,ADDR sErrCode
		pop	ecx
		inv	udw2str,ecx,eax
		pop	edi
		pop	edx
		pop	ecx
		ret
PrepareErrorMsg	endp
;-------------------------------

;Name:		Concat
;Description:	concatenates a string at pMsg2 to a string at pMsg1
;In:	pMsg1	pointer to a message #1
;	pMsg2	pointer to a message #2
;Out:	none
Concat		proc	dest,src
		pushfd
		push	ecx
		push	esi
		push	edi
	;looking for 0
		xor	eax,eax
		xor	ecx,ecx
		mov	edi,dest
		mov	esi,src
		cld
		repne	scasb
	;storing src
	@@:	lodsb
		stosb
		cmp	al,0
		jne	@b
		mov	eax,edi
		dec	eax
		pop	edi
		pop	esi
		pop	ecx
		popfd
		ret
Concat		endp
;-------------------------------

;Name:		AllocEx
;Description:	allocates virtual memory with r/w permissions and fills it w/ 0s
;		On error, displays extended error information and terminates the program
;In:	hWin	handle to the current window
;	pBuf	pointer to Buf structure
;	pMsg	pointer to a message to be displayed on error (2nd argument of FatalIfEx)
;Out:	none
AllocEx		proc	hWin,pBuf,pMsg
		push	eax
		push	ecx
		mov	eax,pBuf
		push	eax
		add	eax,4
		mov	eax,[eax]	;size member of Buf structure
		inv	VirtualAlloc,0,eax,MEM_RESERVE or MEM_COMMIT,PAGE_READWRITE
		pop	ecx
		FatalIfEx hWin,0,ADDR eMemAlloc2,pMsg
		mov	[ecx],eax	;ptr member is set
		pop	ecx
		pop	eax
		ret
AllocEx		endp
;-------------------------------

;Name:		GetCmdLineParams
;Description:	parses the command line
;In:	num	maximum number of parameters to parse
;	p	pointer to num*4 bytes long array to store the pointers to the params
;Out:	eax	number of parameters actually analyzed
GetCmdLineParams proc	num,p
		push	ecx
		push	edx
		call	GetCommandLine
		mov	ecx,p
		xor	edx,edx
		cmp	byte ptr [eax],34
		je	@@qtn
	@@:	inc	eax
		cmp	byte ptr [eax],32
		ja	@b
		jmp	@@loop
	@@qtn:	inc	eax
		cmp	byte ptr [eax],34
		jne	@@qtn
	@@loop:	inc	eax
		cmp	byte ptr [eax],0
		je	@@done
		cmp	byte ptr [eax],32
		jbe	@@loop
		cmp	byte ptr [eax],34
		je	@@qt
		mov	[ecx],eax
		add	ecx,4
		inc	edx
	@@:	inc	eax
		cmp	byte ptr [eax],0
		je	@@done
		cmp	byte ptr [eax],34
		je	@@put0
		cmp	byte ptr [eax],32
		ja	@b
	@@put0:	mov	byte ptr [eax],0
		cmp	edx,num
		je	@@done
		jmp	@@loop
		
	@@qt:	inc	eax
		cmp	byte ptr [eax],0
		je	@@done
		cmp	byte ptr [eax],34
		je	@@loop
		cmp	byte ptr [eax],32
		jbe	@@loop
		mov	[ecx],eax
		add	ecx,4
		inc	edx
	@@:	inc	eax
		cmp	byte ptr [eax],0
		je	@@done
		cmp	byte ptr [eax],34
		jne	@b
		jmp	@@put0
	@@done:	mov	eax,edx
		pop	edx
		pop	ecx
		ret
GetCmdLineParams endp

end		Start